/*
  implements a node (n sources, m devices)
  Copyright (C) 1998  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package

 */




#ifndef __NODEDEVICE_H
#define __NODEDEVICE_H

#include <deviceConfig/deviceConfigArray.h>
#include <graph/yafGraph.h>


#include <pthread.h>
#include <qobject.h>

extern "C" {
#include <unistd.h>
	   }

/**
  configuring a nodeDevice.
  
  <p>
  Every nodeDevice holds a DeviceConfig object. This
  configuration represent the current configuration of
  the stream. The device may have other configuration options
  but they may not belong to the stream. E.g: equalizerDevice which
  changes the stream frequencies. This is not important for the
  stream which frequency is changed.
  <p>

  The deviceConfig may be set to useful default values in the constructor.
  Then, every device has a Variabla lAutoConfig.
  If this is true the device calls the applyConfig(..) method
  if the streamData config and the current Config in the
  device differs from each other.
  <p>
  A user can override these configuration.
  Sometimes it may not be possible to make a good autoConfiguartion.
  <p>
  E.g A mixerDevice is entered by more streamProducer. If
  one thread delivers 22,5 Khz and another 44,1 Khz the mixer
  cannot automatically transform the 44,1->22,5 or 22,5->44,1
  because it does not now how to do  this. But if the user
  select a mixerWithFilterDevice he can configure it manually
  to downsample the 44,1Khz data. A good default for the
  filter may be : mix to lowset frequency But this configuration
  does not belong to the stream. Its an external automatic configuartion.
  <p>
  The mixer can only mix data. If the user connect these streamProducers
  with an incompatible format together he should know what he
  is doing!

*/
  

class NodeDevice : public QObject {
 Q_OBJECT

  struct YafGraph* yafGraph;
  struct Edges* sources;
  struct Edges* listeners;

  pthread_mutex_t writeOutMut;
  pthread_mutex_t writeInMut;


  struct DeviceConfig* currentConfig;

 public:
  NodeDevice();
  virtual ~NodeDevice();

  void addListener(struct NodeDevice *);
  void removeListener(struct NodeDevice *);
  int hasListener();

  struct Edges* getListeners();
  struct Edges* getSources();

  virtual void writeIn(struct NodeDevice* source,struct DeviceConfig* config);
  virtual void writeOut(struct DeviceConfigArray* buf);
  
  virtual char* getNodeName();

  /** 
      devices which write in this device must request this lock
      until they are ready
  */
  void writeInLock();
  void writeInUnlock();
  
  /**
    this mut is locked when the thread is active (if there is any)
  */
  virtual void writeOutLock();
  virtual void writeOutUnlock();

  /* If I would include dasiyChain.h the compiler returns an error
     because auf a recursive inclusion.
     the void pointer can safley casted to a dasiyChain
  */
  void* getDaisyChain();        

  void lockDeviceTree();
  void unlockDeviceTree();


  /**
    these method must be thread safe!
  */
  virtual void setDeviceConfig(struct DeviceConfig* aConfig);
  struct DeviceConfig* getDeviceConfig();


 private:
  void addSource(struct NodeDevice* source);
  void removeSource(struct NodeDevice* source); 

};

#endif
  
  



